@@ -22,7 +22,7 @@ VERSION   = "0.2.1"
 # NOTE: For a single contributor, use: ("Contributor Name",)
 CONTRIBUTORS = ()
 
-PROVIDERS = ("OpenAI", "Google", "Groq", "Anthropic")
+PROVIDERS = ("xAI", "OpenAI", "Google", "Groq", "Anthropic")
 
 # HoloAI framework development message
 def addNames(items):
@@ -151,24 +151,11 @@ def getContextLength(model_path):
         return 512
 
 
-# def parseInstructions(kwargs):
-#     """
-#     Combines 'system' and 'instructions' in kwargs (if both), system first.
-#     If only one is present, returns that one.
-#     Returns None if neither present.
-#     """
-#     system = kwargs.get("system")
-#     instructions = kwargs.get("instructions")
-#     if system and instructions:
-#         # system = f"Main Instructions:\n {system}\n\nSub Instructions:\n {instructions}"
-#         system = f"{system}\n\n{instructions}"
-#         return system
-#     return system or instructions
-
 def parseInstructions(kwargs):
     """
     Combines 'system' and 'instructions' in kwargs with labeled headers if both are present,
     'system' first. If only one is present, returns that one as a string.
+    Always runs the result through systemInstructions before returning.
     Returns None if neither is present.
     """
     system = kwargs.get("system")
@@ -178,8 +165,37 @@ def parseInstructions(kwargs):
             "Main": system,
             "Sub": instructions
         }
-        return "\n".join(f"{key} Instructions:\n{value.strip()}" for key, value in sections.items())
-    return system or instructions
+        combined = "\n".join(f"{key} Instructions:\n{value.strip()}" for key, value in sections.items())
+        return _formatSystemInstructions(combined)
+    if system or instructions:
+        return _formatSystemInstructions(system or instructions)
+    return None
+
+
+def _formatSystemInstructions(system):
+    """
+    Returns the full system message for the model, combining dev message and user system instructions.
+    Handles structured and non-structured formats.
+    """
+    devMessage = DEV_MSG
+    if not system:
+        return devMessage
+    if isStructured(system):
+        systemContents = "\n".join(item['content'] for item in system)
+        return f"{devMessage}\n{systemContents}"
+    return f"{devMessage}\n{system}"
+
+
+def validateResponseArgs(model, user):
+    if not model:
+        raise ValueError("Model cannot be None or empty.")
+    if not user:
+        raise ValueError("User input cannot be None or empty.")
+
+def validateVisionArgs(model, user, paths):
+    validateResponseArgs(model, user)
+    if not paths or not isinstance(paths, list) or len(paths) == 0:
+        raise ValueError("paths must be a list with at least one item.")
 
 
 def parseModels(models):
@@ -544,3 +560,39 @@ def unsupportedFormat(ext):
     Raises a ValueError for unsupported file formats.
     """
     raise ValueError(f"File format '{ext}' is not supported for Vision frame extraction")
+
+
+ANTHROPIC_MODELS = (
+    "claude-3-7",
+    "claude-sonnet-4",
+    "claude-opus-4",
+)
+
+GOOGLE_MODELS = (
+    "gemini-2.5",
+)
+
+OPENAI_MODELS = (
+    "o",  # OpenAI
+)
+
+GROQ_MODELS = (
+    "qwen/qwen3-32b",
+    "deepseek-r1-distill-llama-70b",
+)
+
+XAI_MODELS = (
+    "grok-3-mini",
+)
+
+REASONING_MODELS = (
+    *ANTHROPIC_MODELS,
+    *GOOGLE_MODELS,
+    *OPENAI_MODELS,
+    *GROQ_MODELS,
+    *XAI_MODELS,
+)
+
+def supportsReasoning(model: str) -> bool:
+    """Return True if the model supports reasoning/thinking config."""
+    return model.lower().startswith(REASONING_MODELS)
\ No newline at end of file
