@@ -1,7 +1,56 @@
 ﻿
+# import logging
+# import os
+# import re
+# import threading
+# from datetime import datetime
+# from dotenv import load_dotenv
+
+# from .HAIUtils.HAIUtils import (
+#     getFrameworkInfo,
+#     formatJsonInput,
+#     formatJsonExtended,
+#     parseJsonInput,
+#     formatTypedInput,
+#     formatTypedExtended,
+#     parseTypedInput,
+#     parseInstructions,
+#     parseModels,
+#     isStructured,
+#     safetySettings,
+#     extractImagePaths,
+#     getFrames
+# )
+
+# load_dotenv()
+# logger = logging.getLogger(__name__)
+
+# def isKeySet(envKey):
+#     return os.getenv(envKey) is not None
+
+# PROVIDERS = {
+#     "OPENAI_API_KEY":    (".HAIConfigs.OpenAIConfig",   "OpenAIConfig",   "openai"),
+#     "ANTHROPIC_API_KEY": (".HAIConfigs.AnthropicConfig","AnthropicConfig","anthropic"),
+#     "GOOGLE_API_KEY":    (".HAIConfigs.GoogleConfig",   "GoogleConfig",   "google"),
+#     "GROQ_API_KEY":      (".HAIConfigs.GroqConfig",     "GroqConfig",     "groq"),
+# }
+
+# providerMap = {}
+
+# for key, (module, clsName, mapKey) in PROVIDERS.items():
+#     if isKeySet(key):
+#         try:
+#             mod = __import__(module, fromlist=[clsName])
+#             providerMap[mapKey] = getattr(mod, clsName)()
+#         except ImportError as e:
+#             # Optionally log or warn here
+#             pass
+
+import logging
 import os
 import re
 import threading
+from importlib import import_module
 from datetime import datetime
 from dotenv import load_dotenv
 
@@ -22,43 +71,69 @@ from .HAIUtils.HAIUtils import (
 )
 
 load_dotenv()
+logger = logging.getLogger(__name__)
 
-PROVIDER_KEYS = {
-    "openai": "OPENAI_API_KEY",
-    "anthropic": "ANTHROPIC_API_KEY",
-    "google": "GOOGLE_API_KEY",
-    "groq": "GROQ_API_KEY",
-}
+def setProvider(apiInput=None):
+    """
+    Sets provider API keys from string, tuple, or list, or env.
+    Passes API key directly to config if possible.
+    Returns a providerMap of all found providers.
+    """
 
-def isKeySet(envKey):
-    return os.getenv(envKey) is not None
+    PROVIDERS = {
+        "OPENAI_API_KEY":    ("HoloAI.HAIConfigs.OpenAIConfig",   "OpenAIConfig",   "openai"),
+        "ANTHROPIC_API_KEY": ("HoloAI.HAIConfigs.AnthropicConfig","AnthropicConfig","anthropic"),
+        "GOOGLE_API_KEY":    ("HoloAI.HAIConfigs.GoogleConfig",   "GoogleConfig",   "google"),
+        "GROQ_API_KEY":      ("HoloAI.HAIConfigs.GroqConfig",     "GroqConfig",     "groq"),
+        "XAI_API_KEY":       ("HoloAI.HAIConfigs.xAIConfig",      "xAIConfig",      "xai"),
+    }
 
 
-MODEL_PREFIX_MAP = {
-    ("gpt",): "openai", # "o1", "o3"): "openai",
+    # Step 1: Parse keys from apiInput (if given) and always set env too (for backward compat)
+    keyMap = {}
+    if apiInput is None:
+        inputList = []
+    elif isinstance(apiInput, str):
+        inputList = [apiInput.strip()]
+    elif isinstance(apiInput, (list, tuple)):
+        inputList = [s.strip() for s in apiInput if isinstance(s, str) and s.strip()]
+    else:
+        raise ValueError("setProvider input must be a string, list, tuple, or None")
+
+    for assignment in inputList:
+        try:
+            envKey, apiKey = assignment.split('=', 1)
+            envKey = envKey.strip()
+            apiKey = apiKey.strip()
+            os.environ[envKey] = apiKey  # set for backward compat
+            keyMap[envKey] = apiKey      # track for direct pass
+        except Exception:
+            raise ValueError(f"Each assignment must be 'PROVIDER_KEY=key', got: {assignment}")
+
+    providerMap = {}
+    for envKey, (module, clsName, mapKey) in PROVIDERS.items():
+        apiKey = keyMap.get(envKey) or os.getenv(envKey)
+        if apiKey:
+            try:
+                mod = import_module(module)
+                # Pass apiKey directly if config supports it, else fallback to no-arg
+                try:
+                    providerMap[mapKey] = getattr(mod, clsName)(apiKey)
+                except TypeError:
+                    providerMap[mapKey] = getattr(mod, clsName)()
+            except ImportError:
+                continue
+
+    return providerMap
+
+MODELS = {
+    ("gpt", "o"): "openai",
     ("claude",): "anthropic",
-    ("llama", "meta-llama", "gemma2",): "groq",
+    ("llama", "meta-llama", "gemma2", "qwen", "deepseek",): "groq",
     ("gemini", "gemma",): "google",
+    ("grok"): "xai",
 }
 
-providerMap = {}
-
-if isKeySet("OPENAI_API_KEY"):
-    from .HAIConfigs.OpenAIConfig import OpenAIConfig
-    providerMap["openai"] = OpenAIConfig()
-
-if isKeySet("ANTHROPIC_API_KEY"):
-    from .HAIConfigs.AnthropicConfig import AnthropicConfig
-    providerMap["anthropic"] = AnthropicConfig()
-
-if isKeySet("GOOGLE_API_KEY"):
-    from .HAIConfigs.GoogleConfig import GoogleConfig
-    providerMap["google"] = GoogleConfig()
-
-if isKeySet("GROQ_API_KEY"):
-    from .HAIConfigs.GroqConfig import GroqConfig
-    providerMap["groq"] = GroqConfig()
-
 
 class HoloAI:
     _instance = None
@@ -75,7 +150,7 @@ class HoloAI:
         if getattr(self, 'initialized', False):
             return
 
-        self.providerMap = providerMap
+        self.providerMap = setProvider()
 
         self.initialized = True
 
@@ -92,13 +167,25 @@ class HoloAI:
         """
         return list(self.providerMap.keys())
 
+    def setProvider(self, apiInput=None):
+        """
+        Sets provider API keys from:
+        - a single string:      'PROVIDER_KEY=api_key'
+        - a tuple of strings:   ('PROVIDER_KEY=api_key', ...)
+        - a list of strings:    ['PROVIDER_KEY=api_key', ...]
+        - or uses environment if nothing passed.
+
+        Returns a providerMap of all found providers.
+        """
+        self.providerMap = setProvider(apiInput)
+
     def _inferModelProvider(self, model: str):
         """
         Infers the provider based on the model name.
         Returns the provider name as a string, or None if not found.
         """
         return next(
-            (provider for prefixes, provider in MODEL_PREFIX_MAP.items()
+            (provider for prefixes, provider in MODELS.items()
              if any(model.startswith(prefix) for prefix in prefixes)),
             None
         )
@@ -127,48 +214,13 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required).
                 Accepts a single prompt string or a message history (list of messages).
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - skills: (list) Skills to use (Optional).
             - tools: (list) Tools to use (Optional).
-            - tokens: (int) Max tokens to use (Optional, default: 369).
-            - verbose: (bool) Return verbose output (Optional, default: False).
+            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
+            - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
+            - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Response object, or a Vision object if image paths are found.
         """
-        # kwargs  = {k.lower(): v for k, v in kwargs.items()}
-        # models  = kwargs.get("models") or kwargs.get("model")
-        # raw     = kwargs.get("input") or kwargs.get("user")
-        # system  = parseInstructions(kwargs)
-        # verbose = kwargs.get("verbose", False)
-        # if models is None or raw is None:
-        #     raise ValueError("HoloCompletion requires 'model' or 'models' and input/user")
-
-        # # Use parseModels util
-        # models = parseModels(models)
-
-        # # ————— 1) Isolate the *last* user message, not the whole history —————
-        # if isinstance(raw, list):
-        #     last = raw[-1]
-        #     text = last["content"] if isinstance(last, dict) and "content" in last else str(last)
-        # else:
-        #     text = str(raw)
-
-        # # ————— 2) Find any image paths in that single prompt —————
-        # image_paths = extractImagePaths(text)
-
-        # if image_paths:
-        #     img = image_paths[-1]
-        #     prompt_only = re.split(re.escape(img), text)[0].strip()
-        #     return self.Vision(
-        #         model=models['vision'],
-        #         system=system,
-        #         user=prompt_only,
-        #         paths=image_paths,
-        #         collect=5,
-        #         verbose=verbose
-        #     )
-
-        # # If no image, run text/response as usual, using the 'response' model
-        # kwargs['model'] = models['response']
-        # return self.Response(**kwargs)
         return self._routeCompletion(**kwargs)
 
     def HoloAgent(self, **kwargs):
@@ -185,70 +237,31 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required).
                 Accepts a single prompt string or a message history (list of messages).
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - skills: (list) Skills to use (Optional).
             - tools: (list) Tools to use (Optional).
-            - tokens: (int) Max tokens to use (Optional, default: 369).
-            - verbose: (bool) Return verbose output (Optional, default: False).
+            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
+            - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
+            - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Response object, or a Vision object if image paths are found.
         """
-        # kwargs  = {k.lower(): v for k, v in kwargs.items()}
-        # models  = kwargs.get("models") or kwargs.get("model")
-        # raw     = kwargs.get("input") or kwargs.get("user")
-        # system  = parseInstructions(kwargs)
-        # verbose = kwargs.get("verbose", False)
-        # if models is None or raw is None:
-        #     raise ValueError("HoloCompletion requires 'model' or 'models' and input/user")
-
-        # # Use parseModels util
-        # models = parseModels(models)
-
-        # # ————— 1) Isolate the *last* user message, not the whole history —————
-        # if isinstance(raw, list):
-        #     last = raw[-1]
-        #     text = last["content"] if isinstance(last, dict) and "content" in last else str(last)
-        # else:
-        #     text = str(raw)
-
-        # # ————— 2) Find any image paths in that single prompt —————
-        # image_paths = extractImagePaths(text)
-
-        # if image_paths:
-        #     img = image_paths[-1]
-        #     prompt_only = re.split(re.escape(img), text)[0].strip()
-        #     return self.Vision(
-        #         model=models['vision'],
-        #         system=system,
-        #         user=prompt_only,
-        #         paths=image_paths,
-        #         collect=5,
-        #         verbose=verbose
-        #     )
-
-        # # If no image, run text/response as usual, using the 'response' model
-        # kwargs['model'] = models['response']
-        # return self.Response(**kwargs)
         return self._routeCompletion(**kwargs)
 
-    def Agent(self, **kwargs):
+    def Reasoning(self, **kwargs):
         """
-        Get a Response from the Agent model.
+        Get a Response from the Response model.
         :param kwargs: Keyword arguments to customize the request.
             - model: (str) The model to use (Required).
             - system/instructions: (str) System prompt or additional instructions (Optional).
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - skills: (list) Skills to use (Optional).
             - tools: (list) Tools to use (Optional).
-            - tokens: (int) Max tokens to use (Optional (default: 369)).
+            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
+            - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
             - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Response object.
         """
-        #print(f"\n[Response Request] {kwargs}")
-        # kwargs = {k.lower(): v for k, v in kwargs.items()}
-        # model  = kwargs.get('model')
-        # config = self._getProviderConfig(model)
-        # return config.getResponse(**kwargs)
         return self._routeResponse(**kwargs)
 
     def Response(self, **kwargs):
@@ -260,17 +273,11 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - skills: (list) Skills to use (Optional).
             - tools: (list) Tools to use (Optional).
-            - tokens: (int) Max tokens to use (Optional (default: 369)).
+            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
             - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Response object.
         """
-        #print(f"\n[Response Request] {kwargs}")
-        # kwargs = {k.lower(): v for k, v in kwargs.items()}
-        # model  = kwargs.get('model')
-        # config = self._getProviderConfig(model)
-        # return config.getResponse(**kwargs)
         return self._routeResponse(**kwargs)
 
     def Vision(self, **kwargs):
@@ -282,56 +289,45 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - tokens: (int) Max tokens to use (Optional (default: 369)).
+            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
             - paths: (list) List of image paths (default: empty list).
             - collect: (int) Number of frames to collect (default: 10).
             - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Vision response object.
         """
-        #print(f"\n[Vision Request] {kwargs}")
-        kwargs = {k.lower(): v for k, v in kwargs.items()}
-        model  = kwargs.get('model')
-        config = self._getProviderConfig(model)
-        return config.getVision( **kwargs)
+        return self._routeVision(**kwargs)
+
+    def Agent(self, **kwargs):
+        """
+        Get a Response from the Agent model.
+        :param kwargs: Keyword arguments to customize the request.
+            - task: (str) Task type ('response', 'reasoning', 'vision') (Optional (default: 'response')).
+            - model: (str) The model to use (Required).
+            - system/instructions: (str) System prompt or additional instructions (Optional).
+            - user/input: (str or list) The main user input (Required). 
+                Accepts a single prompt string or a message history (list of messages). 
+                Both 'user' and 'input' are interchangeable; use either (Preferred: input).
+            - tools: (list) Tools to use (Optional).
+            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
+            - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
+            - paths: (list) List of image paths (default: empty list).
+            - collect: (int) Number of frames to collect (default: 10).
+            - verbose: (bool) Return verbose output (Optional (default: False)).
+        :return: A Reasoning, Response, or Vision object.
+        """
+        #return self._routeResponse(**kwargs)
+        task = kwargs.get('task', 'response').lower()
+        taskMap = {
+            'reasoning': self._routeResponse,
+            'response': self._routeResponse,
+            'vision': self._routeVision,
+        }
+        if task not in taskMap:
+            raise ValueError(f"Unknown task: '{task}'. Supported tasks: {list(taskMap.keys())}")
+        return taskMap[task](**kwargs)
 
     #------------- Utility Methods -------------#
-    # def _routeCompletion(self, **kwargs):
-    #     kwargs  = {k.lower(): v for k, v in kwargs.items()}
-    #     models  = kwargs.get("models") or kwargs.get("model")
-    #     raw     = kwargs.get("input") or kwargs.get("user")
-    #     system  = parseInstructions(kwargs)
-    #     verbose = kwargs.get("verbose", False)
-    #     if models is None or raw is None:
-    #         raise ValueError("HoloCompletion requires 'model' or 'models' and input/user")
-
-    #     # Use parseModels util
-    #     models = parseModels(models)
-
-    #     # ————— 1) Isolate the *last* user message, not the whole history —————
-    #     if isinstance(raw, list):
-    #         last = raw[-1]
-    #         text = last["content"] if isinstance(last, dict) and "content" in last else str(last)
-    #     else:
-    #         text = str(raw)
-
-    #     # ————— 2) Find any image paths in that single prompt —————
-    #     image_paths = extractImagePaths(text)
-
-    #     if image_paths:
-    #         img = image_paths[-1]
-    #         prompt_only = re.split(re.escape(img), text)[0].strip()
-    #         return self.Vision(
-    #             model=models['vision'],
-    #             system=system,
-    #             user=prompt_only,
-    #             paths=image_paths,
-    #             collect=5,
-    #             verbose=verbose
-    #         )
-
-    #     # If no image, run text/response as usual, using the 'response' model
-    #     kwargs['model'] = models['response']
-    #     return self.Response(**kwargs)
     def _routeCompletion(self, **kwargs):
         kwargs  = {k.lower(): v for k, v in kwargs.items()}
         models  = kwargs.get("models") or kwargs.get("model")
@@ -357,7 +353,7 @@ class HoloAI:
         def visionMode():
             img = imagePaths[-1]
             promptOnly = re.split(re.escape(img), text)[0].strip()
-            return self.Vision(
+            return self._routeVision(
                 model=models['vision'],
                 system=system,
                 user=promptOnly,
@@ -386,6 +382,13 @@ class HoloAI:
         config = self._getProviderConfig(model)
         return config.getResponse(**kwargs)
 
+    def _routeVision(self, **kwargs):
+        #print(f"\n[Vision Request] {kwargs}")
+        kwargs = {k.lower(): v for k, v in kwargs.items()}
+        model  = kwargs.get('model')
+        config = self._getProviderConfig(model)
+        return config.getVision( **kwargs)
+
     def isStructured(self, obj):
         """
         Check if the input is a structured list of message dicts.
