@@ -9,6 +9,7 @@ from SynMem import SynMem as HoloMem
 from SynLrn import SynLrn as HoloLrn
 from BitSig import BitSig as HoloLog
 from MediaCapture import MediaCapture as HoloCapture
+from AgentToAgent import AgentToAgent as HoloRelay
 
 __all__ = [
     "HoloAI",
@@ -18,5 +19,6 @@ __all__ = [
     "HoloLrn",
     "HoloLog",
     "HoloViro",
-    "HoloCapture"
+    "HoloRelay",
+    "HoloCapture",
 ]
